@@ -1,11 +1,11 @@
 /*
-**  SA - OSSP Socket Abstraction Library
+**  OSSP sa - Socket Abstraction
 **  Copyright (c) 2001-2002 Ralf S. Engelschall <rse@engelschall.com>
 **  Copyright (c) 2001-2002 The OSSP Project <http://www.ossp.org/>
 **  Copyright (c) 2001-2002 Cable & Wireless Deutschland <http://www.cw.com/de/>
 **
-**  This file is part of OSSP SA, a socket abstraction library which
-**  can be found at http://www.ossp.org/pkg/sa/.
+**  This file is part of OSSP sa, a socket abstraction library which
+**  can be found at http://www.ossp.org/pkg/lib/sa/.
 **
 **  Permission to use, copy, modify, and distribute this software for
 **  any purpose with or without fee is hereby granted, provided that
@@ -240,10 +240,17 @@ static int sa_inet_pton(int family, const char *strptr, void *addrptr)
     struct in_addr in_val;
 
     if (family == AF_INET) {
+#if defined(HAVE_INET_ATON)
         /* at least for IPv4 we can rely on the old inet_aton(3)
            and for IPv6 inet_pton(3) would exist anyway */
         if (inet_aton(strptr, &in_val) == 0)
             return 0;
+#elif defined(HAVE_INET_ADDR)
+        /* at least for IPv4 try to rely on the even older inet_addr(3) */
+        memset(&in_val, '\0', sizeof(in_val));
+        if ((in_val.s_addr = inet_addr(strptr)) == ((in_addr_t)-1))
+            return 0;
+#endif
         memcpy(addrptr, &in_val, sizeof(struct in_addr));
         return 1;
     }
@@ -258,14 +265,14 @@ static const char *sa_inet_ntop(int family, const void *src, char *dst, size_t s
 #ifdef HAVE_INET_NTOP
     return inet_ntop(family, src, dst, size);
 #else
-    struct in_addr in_val;
     char *cp;
     int n;
 
     if (family == AF_INET) {
+#ifdef HAVE_INET_NTOA
         /* at least for IPv4 we can rely on the old inet_ntoa(3)
            and for IPv6 inet_ntop(3) would exist anyway */
-        if ((cp = inet_ntoa(src)) == NULL) 
+        if ((cp = inet_ntoa(*((struct in_addr *)src))) == NULL) 
             return NULL;
         n = strlen(cp);
         if (n > size-1)
@@ -273,6 +280,7 @@ static const char *sa_inet_ntop(int family, const void *src, char *dst, size_t s
         memcpy(dst, cp, n);
         dst[n] = '\0';
         return dst;
+#endif
     }
     errno = EAFNOSUPPORT;
     return NULL;
@@ -382,13 +390,13 @@ static int sa_mvsnprintf(char *buffer, size_t bufsize, const char *format, va_li
     else {
         /* perform real output */
         ctx.bufptr = buffer;
-        ctx.buflen = bufsize - 1;
+        ctx.buflen = bufsize;
         n = sa_mvxprintf(sa_mvsnprintf_cb, &ctx, format, ap);
+        if (n != -1 && ctx.buflen == 0)
+            n = -1;
+        if (n != -1)
+            *(ctx.bufptr) = '\0';
     }
-    if (n != -1 && ctx.buflen == 0)
-        n = -1;
-    if (n != -1)
-        *(ctx.bufptr) = '\0';
     return n;
 }
 
@@ -814,8 +822,8 @@ sa_rc_t sa_addr_match(sa_addr_t *saa1, sa_addr_t *saa2, int prefixlen)
         ucp1 = (const unsigned char *)&(((struct sockaddr_in6 *)saa1->saBuf)->sin6_addr);
         ucp2 = (const unsigned char *)&(((struct sockaddr_in6 *)saa2->saBuf)->sin6_addr);
         nBits = 128;
-#endif
     }
+#endif
     else
         return SA_RC(SA_ERR_INT);
 
@@ -862,14 +870,14 @@ static sa_rc_t sa_socket_settimeouts(sa_t *sa)
     if (sa->fdSocket != -1) {
         if (SA_TVISZERO(sa->tvTimeout[SA_TIMEOUT_READ])) {
             if (setsockopt(sa->fdSocket, SOL_SOCKET, SO_RCVTIMEO,
-                           &sa->tvTimeout[SA_TIMEOUT_READ],
-                           sizeof(sa->tvTimeout[SA_TIMEOUT_READ])) < 0)
+                           (const void *)(&sa->tvTimeout[SA_TIMEOUT_READ]),
+                           (socklen_t)(sizeof(sa->tvTimeout[SA_TIMEOUT_READ]))) < 0)
                 return SA_RC(SA_ERR_SYS);
         }
         if (SA_TVISZERO(sa->tvTimeout[SA_TIMEOUT_WRITE])) {
             if (setsockopt(sa->fdSocket, SOL_SOCKET, SO_SNDTIMEO,
-                           &sa->tvTimeout[SA_TIMEOUT_WRITE],
-                           sizeof(sa->tvTimeout[SA_TIMEOUT_WRITE])) < 0)
+                           (const void *)(&sa->tvTimeout[SA_TIMEOUT_WRITE]),
+                           (socklen_t)(sizeof(sa->tvTimeout[SA_TIMEOUT_WRITE]))) < 0)
                 return SA_RC(SA_ERR_SYS);
         }
     }
@@ -1150,7 +1158,7 @@ sa_rc_t sa_option(sa_t *sa, sa_option_t id, ...)
                 break;
             }
             if (setsockopt(sa->fdSocket, IPPROTO_TCP, TCP_NODELAY, 
-                           (void *)&mode, sizeof(mode)) < 0)
+                           (const void *)&mode, (socklen_t)sizeof(mode)) < 0)
                 rv = SA_ERR_SYS;
 #else
             rv = SA_ERR_IMP;
@@ -1165,7 +1173,7 @@ sa_rc_t sa_option(sa_t *sa, sa_option_t id, ...)
                 break;
             }
             if (setsockopt(sa->fdSocket, SOL_SOCKET, SO_LINGER, 
-                           linger, sizeof(struct linger)) < 0)
+                           (const void *)linger, (socklen_t)sizeof(struct linger)) < 0)
                 rv = SA_ERR_SYS;
 #else
             rv = SA_ERR_IMP;
@@ -1192,7 +1200,7 @@ sa_rc_t sa_option(sa_t *sa, sa_option_t id, ...)
                 default: flag = 0; break;
             }
             if (setsockopt(sa->fdSocket, SOL_SOCKET, flag, 
-                           (void *)&mode, sizeof(mode)) < 0)
+                           (const void *)&mode, (socklen_t)sizeof(mode)) < 0)
                 rv = SA_ERR_SYS;
             break;
         }
@@ -1349,7 +1357,7 @@ sa_rc_t sa_connect(sa_t *sa, sa_addr_t *raddr)
 
         /* fetch pending error */
         len = sizeof(error);
-        if (getsockopt(sa->fdSocket, SOL_SOCKET, SO_ERROR, &error, &len) < 0)
+        if (getsockopt(sa->fdSocket, SOL_SOCKET, SO_ERROR, (void *)&error, &len) < 0)
             error = errno;
 
         done:
