@@ -43,7 +43,7 @@
 
 /* fallback for POSIX socklen_t */
 #if defined(HAVE_CONFIG_H) && !defined(HAVE_SOCKLEN_T)
-#define socklen_t int
+typedef int socklen_t;
 #endif
 
 /* embedding support */
